@@ -50,7 +50,7 @@ rdflib.plugin.register(
 
 # Type codes and mapping for RDF and query variable p[arsing and serializing
 
-RDFTYP = ["RDFXML","N3","TURTLE","NT","JSONLD"]
+RDFTYP = ["RDFXML","N3","TURTLE","NT","JSONLD","RDFA"]
 VARTYP = ["JSON","CSV","XML"]
 
 RDFTYPPARSERMAP = (
@@ -59,6 +59,7 @@ RDFTYPPARSERMAP = (
     , "TURTLE": "n3"
     , "NT":     "nt"
     , "JSONLD": "jsonld"
+    , "RDFA":   "rdfa"
     })
 
 RDFTYPSERIALIZERMAP = (
@@ -96,6 +97,34 @@ def formatBindings(template, bindings):
         formatdict[var+"_repr"] = vf%val
     return template%formatdict
 
+# Helper function for CSV formatting query result from JSON
+
+def char_escape(c):
+    if c == '"': return '""'
+    if ord(c) >= 128: return r"\u" + "%04x"%ord(c)
+    return c
+
+def termToCSV(result): 
+    if result == None: 
+        return None
+    resval = result['value']
+    restyp = result['type']
+    if restyp == "uri":
+        return "<" + resval + ">"
+    if restyp == "bnode":
+        return "_:" + resval
+    # strval  = '"' + resval.replace('"', '""') + '"'
+    strval  = '"' + "".join([char_escape(c) for c in resval]) + '"'
+    strlang = result.get('xml:lang', None)
+    if restyp == "literal":
+        if strlang:
+            return strval + '@' + strlang
+        else:
+            return strval
+    if restyp == "typed-literal":
+        return strval + '^^' + result['datatype']
+    raise rdflib.query.ResultException('Unknown term type: %s (%s)'%(term, type(term)))
+
 # Helper functions for JSON formatting and parsing
 # Mostly copied from rdflib SPARQL code (rdfextras/sparql/results/jsonresults)
 
@@ -245,7 +274,8 @@ def getPrefixes(options):
         """
     #    PREFIX xml:        <http://www.w3.org/XML/1998/namespace>
     configbase = os.path.expanduser("~")
-    prefixUri  = options.prefix or resolveUri(".asqc-prefixes", "file://", configbase)
+    prefixUri  = options.prefix or resolveUri(
+        ".asqc-prefixes", "file://", configbase)
     prefixes   = retrieveUri(prefixUri)
     return prefixes or defaultPrefixes
 
@@ -398,6 +428,14 @@ def outputResult(progname, options, result):
             outstr.write("\n")
         elif options.format_var_out == "XML":
             writeResultsXML(outstr, result)
+        elif options.format_var_out == "CSV":
+            qvars = result["head"]["vars"]
+            outstr.write(", ".join(qvars))
+            outstr.write("\n")
+            for bindings in result["results"]["bindings"]:
+                vals = [ termToCSV(bindings[v]) for v in qvars ]
+                outstr.write(", ".join(vals))
+                outstr.write("\n")
         else:
             for bindings in result["results"]["bindings"]:
                 formattedrow = formatBindings(options.format_var_out, bindings)
@@ -457,7 +495,7 @@ def parseCommandArgs(argv):
                        "  %prog [options] [query]\n"+
                        "  %prog --help      for an options summary\n"+
                        "  %prog --examples  to display the path containing example queries"),
-                description="A sparql query client, designed to be used as a filter in a command pieline. "+
+                description="A sparql query client, designed to be used as a filter in a command pipeline. "+
                             "Pipelined data can be RDF or query variable binding sets, depending on the options used.",
                 version="%prog "+asqc_settings.VERSION)
     parser.add_option("--examples",
@@ -479,8 +517,9 @@ def parseCommandArgs(argv):
     parser.add_option("-f", "--format",
                       dest="format",
                       default=None,
-                      help="Format for input and/or output: RDFXML/N3/NT/TURTLE/JSONLD/JSON/CSV/template.  "+
-                           "XML, N3, NT, TURTLE, JSONLD apply to RDF data, others apply to query variable bindings.  "+
+                      help="Format for input and/or output: RDFXML/N3/NT/TURTLE/JSONLD/RDFA/JSON/CSV/template.  "+
+                           "XML, N3, NT, TURTLE, JSONLD, RDFA apply to RDF data, "+
+                           "others apply to query variable bindings.  "+
                            "Multiple comma-separated values may be specified; "+
                            "they are applied to RDF or variable bindings as appropriate.  "+
                            "'template' is a python formatting template with '%(var)s' for query variable 'var'.  "+
@@ -525,7 +564,7 @@ def parseCommandArgs(argv):
     parser.add_option("--format-rdf-in",
                       dest="format_rdf_in",
                       default=None,
-                      help="Format for RDF input data: RDFXML/N3/NT/TURTLE/JSONLD.")
+                      help="Format for RDF input data: RDFXML/N3/NT/TURTLE/JSONLD/RDFA.")
     parser.add_option("--format-rdf-out",
                       dest="format_rdf_out",
                       default=None,
@@ -558,7 +597,8 @@ def parseCommandArgs(argv):
             if fn in RDFTYP:
                 if not options.format_rdf_in:
                     options.format_rdf_in = fn
-                options.format_rdf_out = fn
+                if fn in RDFTYPSERIALIZERMAP:
+                    options.format_rdf_out = fn
             else:
                 if not options.format_var_in and fn in VARTYP:
                     options.format_var_in = fn
